from typing import Optional
import xml.etree.ElementTree as ET
from xml.dom import minidom

from utils.database import DatabaseManager
from utils.errors.export_error import ExportError
from utils.errors.no_export_entity_error import NoExportEntityError

from models.book import Book

class BookExporter():
    def save_xml(self, file_path: str):

        xml = self._get_full_xml()

        if xml is None:
            raise NoExportEntityError("No books found to export")

        with open(file_path, "w", encoding="utf-8") as file:
                file.write(xml)


    def _get_full_xml(self) -> Optional[str]:
        root = ET.Element("books")

        with DatabaseManager().get_session() as session:
            self.books = session.query(Book).all()
        
            if not self.books:
                return None

            for book in self.books:
                # Create a <book> element
                book_element = ET.SubElement(root, "book")
                
                # Add <title>
                title_element = ET.SubElement(book_element, "title")
                title_element.text = book.title
                
                # Add <description>
                description_element = ET.SubElement(book_element, "description")
                description_element.text = book.description
                
                # Add <year_published>
                year_published_element = ET.SubElement(book_element, "year_published")
                year_published_element.text = book.year_published
                
                # Add <isbn>
                isbn_element = ET.SubElement(book_element, "isbn")
                isbn_element.text = book.isbn
                
                # Add <categories>
                categories_element = ET.SubElement(book_element, "categories")
                for category in book.categories:
                    category_element = ET.SubElement(categories_element, "category")
                    category_element.text = category.name
        
        # Convert the tree to a string
        tree_str = ET.tostring(root, encoding="unicode")
        
        # Pretty print the XML
        pretty_xml = minidom.parseString(tree_str).toprettyxml(indent=(" " * 4))
        return pretty_xml